import json
import os
import uuid
import zipfile
import tkinter as tk
from tkinter import ttk, filedialog, simpledialog, messagebox
import requests
import tempfile
import shutil
import re

# --- קבועים ---
MANIFEST_FILE_NAME = "store_manifest.json" # שונה ל-JSON!
APKS_DIR_NAME = "apks"
IMAGES_DIR_NAME = "images"

# --- מחלקת עזר לנתוני אפליקציה ---
class AppEntry:
    def __init__(self, apk_path_or_url, is_url=False):
        self.is_url = is_url
        self.apk_source = apk_path_or_url # נתיב מקורי או URL
        self.temp_local_apk_path = None # נתיב לקובץ APK זמני אם יובא מ-ZIP
        self.temp_local_image_path = None # נתיב לקובץ תמונה זמני אם יובא מ-ZIP
        
        if is_url:
            try:
                # נסו למצוא שם קובץ הגיוני מה-URL
                parsed_url = requests.utils.urlparse(apk_path_or_url)
                path_basename = os.path.basename(parsed_url.path)
                if path_basename and '.' in path_basename: # ודא שיש סיומת, למשל .apk
                    self.apk_file_name = path_basename
                else: # אם אין שם קובץ ברור, צור אחד
                    self.apk_file_name = f"app_from_url_{uuid.uuid4().hex[:8]}.apk"
            except Exception: # אם יש שגיאה בניתוח ה-URL
                self.apk_file_name = f"app_from_url_{uuid.uuid4().hex[:8]}.apk"
            self.app_name_var = tk.StringVar(value=os.path.splitext(self.apk_file_name)[0])
        else: # מקור מקומי (יכול להיות הנתיב המקורי או נתיב זמני לאחר ייבוא)
            # אם apk_path_or_url הוא None או ריק, צור שם קובץ גנרי. זה קורה בטעינה מ-ZIP בהתחלה
            self.apk_file_name = os.path.basename(apk_path_or_url) if apk_path_or_url else f"local_app_{uuid.uuid4().hex[:8]}.apk"
            self.app_name_var = tk.StringVar(value=os.path.splitext(self.apk_file_name)[0])

        self.package_name_var = tk.StringVar(value=self._generate_package_name())
        self.description_var = tk.StringVar(value=f"תיאור עבור {self.app_name_var.get()}")
        self.image_path_var = tk.StringVar(value="") # נתיב מקומי מלא לתמונה המוצגת כרגע
        self.image_file_name_in_zip_var = tk.StringVar(value="") # רק שם הקובץ של התמונה כפי שיישמר ב-ZIP
        self.file_size_var = tk.StringVar(value="0 B")

    def _generate_package_name(self):
        # מנקה תווים לא חוקיים ומוסיף קידומת
        safe_app_name = re.sub(r'[^a-zA-Z0-9_.]', '', self.app_name_var.get().lower().replace(" ", "_"))
        if not safe_app_name: safe_app_name = "unknownapp"
        
        # ודא שהשם לא מתחיל במספר או נקודה
        if safe_app_name and (safe_app_name[0].isdigit() or safe_app_name[0] == '.'):
            safe_app_name = "pkg_" + safe_app_name
        
        # הוספת קידומת com.generated. אם זה עדיין לא נראה כמו package name מלא
        # ניתן להוסיף בדיקות נוספות כאן, למשל אם מכיל נקודות בכלל
        if not safe_app_name.startswith("com.") and not safe_app_name.startswith("org.") and not safe_app_name.startswith("net."):
             safe_app_name = f"com.generated.{safe_app_name}"
        
        # הוסף סיומת ייחודית כדי למנוע התנגשויות (UUID קצר)
        return f"{safe_app_name}.{uuid.uuid4().hex[:6]}"


    def to_dict(self):
        size_str = self.file_size_var.get()
        size_bytes = 0
        try: # המרה חזרה לבייטים
            num_part = float(re.findall(r"[\d\.]+", size_str)[0])
            if "KB" in size_str.upper(): size_bytes = int(num_part * 1024)
            elif "MB" in size_str.upper(): size_bytes = int(num_part * 1024 * 1024)
            elif "B" in size_str.upper(): size_bytes = int(num_part)
        except (ValueError, IndexError): size_bytes = 0 # אם יש שגיאה, ברירת מחדל 0
        
        # חשוב: imageFileName צריך להיות רק שם הקובץ של התמונה, לא הנתיב המלא
        img_fn_in_zip = self.image_file_name_in_zip_var.get()

        return {
            "name": self.app_name_var.get(),
            "description": self.description_var.get(),
            "apkFileName": self.apk_file_name, # זה תמיד יהיה שם הקובץ
            "imageFileName": img_fn_in_zip if img_fn_in_zip else None,
            "packageName": self.package_name_var.get(),
            # שדה apkUrl ישמש רק אם זה באמת אפליקציה מ-URL. 
            # אם זו אפליקציה מקומית (או כזו שיובאה מ-ZIP), הוא יהיה None.
            "apkUrl": self.apk_source if self.is_url else None,
            "fileSize": size_bytes # כערך מספרי (בייטים)
        }

    def update_image_selection(self, local_image_path): # מקבל נתיב מקומי מלא
        self.image_path_var.set(local_image_path)
        # שם הקובץ ל-ZIP תמיד יהיה מבוסס על הנתיב המקומי הזה
        self.image_file_name_in_zip_var.set(os.path.basename(local_image_path) if local_image_path else "")


# --- מחלקת יישום ראשית ---
class BackupCreatorApp:
    def __init__(self, root_window):
        self.root = root_window
        root_window.title("חנות אפליקציות כשרה - יוצר גיבוי (V2)")
        root_window.geometry("850x750")
        self.selected_apks_data = []
        self.current_temp_dir = None # חשוב לאתחל
        
        # וודא שהתיקייה הזמנית נוצרת ונמחקת בסיום היישום
        self.root.protocol("WM_DELETE_WINDOW", self.on_closing)

        paned_window = ttk.PanedWindow(root_window, orient=tk.HORIZONTAL)
        paned_window.pack(fill=tk.BOTH, expand=True, padx=10, pady=10)

        left_pane = ttk.Frame(paned_window, padding="10")
        paned_window.add(left_pane, weight=2)

        settings_frame = ttk.LabelFrame(left_pane, text="הגדרות כלליות", padding="10")
        settings_frame.pack(pady=10, fill="x")
        self._create_settings_widgets(settings_frame)

        apks_list_frame = ttk.LabelFrame(left_pane, text="אפקים לכלול / עריכה", padding="10")
        apks_list_frame.pack(pady=10, fill="both", expand=True)
        self._create_apk_list_widgets(apks_list_frame)

        right_pane = ttk.Frame(paned_window, padding="10")
        paned_window.add(right_pane, weight=3)

        self.selected_apk_details_frame = ttk.LabelFrame(right_pane, text="ערוך פרטי אפק נבחר", padding="10")
        self.selected_apk_details_frame.pack(fill="both", expand=True)
        self._create_apk_details_widgets(self.selected_apk_details_frame)

        action_buttons_frame = ttk.Frame(root_window, padding="10")
        action_buttons_frame.pack(fill="x")
        self._create_action_buttons(action_buttons_frame)

        self.clear_apk_details_fields()

    def on_closing(self):
        """פונקציה המופעלת כאשר חלון היישום נסגר."""
        self._cleanup_temp_dir()
        self.root.destroy()

    def _cleanup_temp_dir(self):
        """מנקה את התיקייה הזמנית אם קיימת."""
        if self.current_temp_dir and os.path.exists(self.current_temp_dir):
            try:
                shutil.rmtree(self.current_temp_dir)
                print(f"DEBUG: תיקייה זמנית נמחקה: {self.current_temp_dir}")
                self.current_temp_dir = None # אפס את הנתיב לאחר המחיקה
            except OSError as e:
                print(f"שגיאה במחיקת תיקייה זמנית {self.current_temp_dir}: {e}")

    def _create_settings_widgets(self, parent_frame):
        self.password_var = tk.StringVar(value="yossi6547")
        self.normal_install_var = tk.BooleanVar(value=False)
        self.use_root_var = tk.BooleanVar(value=True)
        self.root_method_var = tk.StringVar(value="pm_install")
        self.install_location_var = tk.StringVar(value="system/app")
        self.send_feedback_var = tk.BooleanVar(value=True)

        r = 0
        ttk.Label(parent_frame, text="סיסמת חנות:").grid(row=r, column=0, sticky="w", padx=5, pady=2)
        ttk.Entry(parent_frame, textvariable=self.password_var, width=30).grid(row=r, column=1, sticky="ew", padx=5, pady=2)
        r += 1

        ttk.Checkbutton(parent_frame, text="התקנה רגילה", variable=self.normal_install_var).grid(row=r, column=0, columnspan=2, sticky="w", padx=5, pady=2)
        r += 1

        ttk.Checkbutton(parent_frame, text="התקנת רוט", variable=self.use_root_var).grid(row=r, column=0, columnspan=2, sticky="w", padx=5, pady=2)
        r += 1

        ttk.Label(parent_frame, text="שיטת רוט:").grid(row=r, column=0, sticky="w", padx=5, pady=2)
        ttk.Radiobutton(parent_frame, text="PM", variable=self.root_method_var, value="pm_install").grid(row=r, column=1, sticky="w", padx=5, pady=0)
        r += 1
        ttk.Radiobutton(parent_frame, text="Copy", variable=self.root_method_var, value="copy_to_system").grid(row=r, column=1, sticky="w", padx=5, pady=0)
        r += 1

        ttk.Label(parent_frame, text="מיקום העתקה:").grid(row=r, column=0, sticky="w", padx=5, pady=2)
        ttk.Radiobutton(parent_frame, text="/system/app", variable=self.install_location_var, value="system/app").grid(row=r, column=1, sticky="w", padx=5, pady=0)
        r += 1
        ttk.Radiobutton(parent_frame, text="/system/priv-app", variable=self.install_location_var, value="system/priv-app").grid(row=r, column=1, sticky="w", padx=5, pady=0)
        r += 1

        ttk.Checkbutton(parent_frame, text="אפשר משוב", variable=self.send_feedback_var).grid(row=r, column=0, columnspan=2, sticky="w", padx=5, pady=2)
        r += 1

        ttk.Label(parent_frame, text="אודות:").grid(row=r, column=0, sticky="nw", padx=5, pady=2)
        self.about_text_widget = tk.Text(parent_frame, height=4, width=30, wrap=tk.WORD)
        self.about_text_widget.grid(row=r, column=1, sticky="ew", padx=5, pady=2)
        self.about_text_widget.insert(tk.END, "חנות כשרה.")
        parent_frame.columnconfigure(1, weight=1)


    def _create_apk_list_widgets(self, parent_frame):
        self.apk_listbox = tk.Listbox(parent_frame, selectmode=tk.SINGLE, height=10, exportselection=False)
        self.apk_listbox.pack(side=tk.LEFT, fill="both", expand=True, padx=(0, 5))
        self.apk_listbox.bind("<<ListboxSelect>>", self.on_apk_select_from_list)
        apk_scrollbar = ttk.Scrollbar(parent_frame, orient="vertical", command=self.apk_listbox.yview)
        apk_scrollbar.pack(side=tk.LEFT, fill="y")
        self.apk_listbox.config(yscrollcommand=apk_scrollbar.set)

        subframe = ttk.Frame(parent_frame)
        subframe.pack(side=tk.BOTTOM, fill="x", pady=(5, 0))
        ttk.Button(subframe, text="הוסף מקומי", command=self.add_local_apks_handler).pack(side=tk.LEFT, padx=2, pady=2)
        ttk.Button(subframe, text="הוסף מ-URL", command=self.add_url_apk_handler).pack(side=tk.LEFT, padx=2, pady=2)
        ttk.Button(subframe, text="הסר נבחר", command=self.remove_selected_apk_handler).pack(side=tk.LEFT, padx=2, pady=2)


    def _create_apk_details_widgets(self, parent_frame):
        self.selected_apk_source_label = ttk.Label(parent_frame, text="מקור:", wraplength=350)
        self.selected_apk_source_label.pack(anchor="w", pady=2, fill="x")

        ttk.Label(parent_frame, text="שם אפליקציה:").pack(anchor="w", pady=(5, 0))
        self.selected_apk_appname_entry = ttk.Entry(parent_frame, width=40)
        self.selected_apk_appname_entry.pack(anchor="w", fill="x", pady=2)
        self.selected_apk_appname_entry.bind("<KeyRelease>", self.update_selected_apk_details_from_fields)

        ttk.Label(parent_frame, text="שם חבילה:").pack(anchor="w", pady=(5, 0))
        self.selected_apk_package_entry = ttk.Entry(parent_frame, width=40)
        self.selected_apk_package_entry.pack(anchor="w", fill="x", pady=2)
        self.selected_apk_package_entry.bind("<KeyRelease>", self.update_selected_apk_details_from_fields)

        ttk.Label(parent_frame, text="תיאור:").pack(anchor="w", pady=(5, 0))
        self.selected_apk_description_text = tk.Text(parent_frame, height=5, width=40, wrap=tk.WORD)
        self.selected_apk_description_text.pack(anchor="w", fill="both", pady=2, expand=True)
        self.selected_apk_description_text.bind("<KeyRelease>", self.update_selected_apk_details_from_fields)

        img_frame = ttk.Frame(parent_frame)
        img_frame.pack(anchor="w", fill="x", pady=(10, 0))
        self.selected_apk_image_label_var = tk.StringVar(value="תמונה: כלום")
        ttk.Label(img_frame, textvariable=self.selected_apk_image_label_var).pack(side=tk.LEFT, pady=2, padx=(0, 5))
        ttk.Button(img_frame, text="בחר תמונה...", command=self.select_image_for_selected_apk).pack(side=tk.LEFT, pady=2)

        size_frame = ttk.Frame(parent_frame)
        size_frame.pack(anchor="w", fill="x", pady=(10, 0))
        self.selected_apk_size_label_var = tk.StringVar(value="גודל: לא זמין")
        ttk.Label(size_frame, textvariable=self.selected_apk_size_label_var).pack(side=tk.LEFT, pady=2, padx=(0, 5))
        ttk.Button(size_frame, text="חשב גודל (מ-URL)", command=self.calculate_apk_size_handler).pack(side=tk.LEFT, pady=2)


    def _create_action_buttons(self, parent_frame):
        ttk.Button(parent_frame, text="צור גיבוי ZIP", command=self.create_backup_zip_handler, style="Accent.TButton").pack(pady=10)
        ttk.Button(parent_frame, text="ייבא ZIP", command=self.load_backup_zip_handler).pack(pady=5)
        style = ttk.Style()
        style.configure("Accent.TButton", font=("Segoe UI", 10, "bold"))


    def add_local_apks_handler(self):
        files = filedialog.askopenfilenames(title="בחר APKs מקומיים", filetypes=(("APK", "*.apk"),))
        if files:
            for f_path in files:
                # בדוק אם הקובץ כבר נוסף (לפי הנתיב המקורי)
                if not any(entry.apk_source == f_path for entry in self.selected_apks_data):
                    app_entry = AppEntry(f_path, is_url=False)
                    try: # קביעת גודל קובץ מיידית
                        size_bytes = os.path.getsize(f_path)
                        app_entry.file_size_var.set(self.format_size(size_bytes))
                    except OSError:
                        app_entry.file_size_var.set("שגיאה בגודל")
                    self.selected_apks_data.append(app_entry)
                    self.apk_listbox.insert(tk.END, app_entry.apk_file_name)
            
            if self.selected_apks_data:
                # בחר את הפריט האחרון שנוסף
                self.apk_listbox.selection_clear(0, tk.END)
                self.apk_listbox.selection_set(self.apk_listbox.size() - 1)
                self.on_apk_select_from_list(None)

    def add_url_apk_handler(self):
        url = simpledialog.askstring("הוסף מ-URL", "הכנס כתובת APK:")
        if url:
            if not any(entry.apk_source == url for entry in self.selected_apks_data):
                app_entry = AppEntry(url, is_url=True)
                self.selected_apks_data.append(app_entry)
                self.apk_listbox.insert(tk.END, app_entry.apk_file_name)
            if self.selected_apks_data:
                self.apk_listbox.selection_clear(0, tk.END)
                self.apk_listbox.selection_set(self.apk_listbox.size() - 1)
                self.on_apk_select_from_list(None)


    def remove_selected_apk_handler(self):
        sel_idx = self.apk_listbox.curselection()
        if sel_idx:
            idx_rem = sel_idx[0]
            entry_rem = self.selected_apks_data[idx_rem]
            
            # ניקוי קבצים זמניים אם קיימים
            if entry_rem.temp_local_apk_path and os.path.exists(entry_rem.temp_local_apk_path):
                try:
                    os.remove(entry_rem.temp_local_apk_path)
                    print(f"DEBUG: נמחק קובץ APK זמני: {entry_rem.temp_local_apk_path}")
                except OSError as e:
                    print(f"שגיאה בניקוי APK זמני: {e}")
            
            if entry_rem.temp_local_image_path and os.path.exists(entry_rem.temp_local_image_path):
                try:
                    os.remove(entry_rem.temp_local_image_path)
                    print(f"DEBUG: נמחקה תמונה זמנית: {entry_rem.temp_local_image_path}")
                except OSError as e:
                    print(f"שגיאה בניקוי תמונה זמנית: {e}")
            
            self.apk_listbox.delete(idx_rem)
            del self.selected_apks_data[idx_rem]
            
            if self.apk_listbox.size() > 0:
                # בחר פריט אחר לאחר המחיקה (אם קיים)
                new_sel_idx = max(0, idx_rem - 1) if idx_rem == self.apk_listbox.size() else idx_rem
                if new_sel_idx < self.apk_listbox.size():
                    self.apk_listbox.selection_set(new_sel_idx)
                self.on_apk_select_from_list(None)
            else:
                self.clear_apk_details_fields()

    def on_apk_select_from_list(self, event):
        sel_idx = self.apk_listbox.curselection()
        if sel_idx:
            idx = sel_idx[0]
            app_entry = self.selected_apks_data[idx]
            
            self.selected_apk_source_label.config(text=f"מקור: {app_entry.apk_source}")
            
            self.selected_apk_appname_entry.config(state=tk.NORMAL)
            self.selected_apk_appname_entry.delete(0, tk.END)
            self.selected_apk_appname_entry.insert(0, app_entry.app_name_var.get())
            
            self.selected_apk_package_entry.config(state=tk.NORMAL)
            self.selected_apk_package_entry.delete(0, tk.END)
            self.selected_apk_package_entry.insert(0, app_entry.package_name_var.get())
            
            self.selected_apk_description_text.config(state=tk.NORMAL)
            self.selected_apk_description_text.delete("1.0", tk.END)
            self.selected_apk_description_text.insert("1.0", app_entry.description_var.get())
            
            # הצג את שם הקובץ בלבד עבור התמונה
            image_display_name = os.path.basename(app_entry.image_path_var.get()) if app_entry.image_path_var.get() else 'כלום'
            self.selected_apk_image_label_var.set(f"תמונה: {image_display_name}")
            
            self.selected_apk_size_label_var.set(f"גודל: {app_entry.file_size_var.get()}")
        else:
            self.clear_apk_details_fields()

    def update_selected_apk_details_from_fields(self, event=None):
        sel_idx = self.apk_listbox.curselection()
        if sel_idx:
            idx = sel_idx[0]
            entry = self.selected_apks_data[idx]
            
            # עדכן את ה-StringVar-ים של האובייקט AppEntry
            entry.app_name_var.set(self.selected_apk_appname_entry.get())
            entry.package_name_var.set(self.selected_apk_package_entry.get())
            entry.description_var.set(self.selected_apk_description_text.get("1.0", tk.END).strip())
            
            self.apk_listbox.selection_set(idx) # ודא שהבחירה נשארת על הפריט הנבחר


    def select_image_for_selected_apk(self):
        sel_idx = self.apk_listbox.curselection()
        if not sel_idx:
            messagebox.showwarning("שגיאה", "בחר אפק.")
            return
        
        idx = sel_idx[0]
        image_path = filedialog.askopenfilename(
            title=f"בחר תמונה עבור {self.selected_apks_data[idx].app_name_var.get()}",
            filetypes=(("תמונות", "*.png *.jpg *.jpeg *.gif"),)
        )
        
        if image_path:
            self.selected_apks_data[idx].update_image_selection(image_path)
            # עדכן את התווית בתצוגה
            self.selected_apk_image_label_var.set(f"תמונה: {os.path.basename(image_path)}")


    def calculate_apk_size_handler(self):
        sel_idx = self.apk_listbox.curselection()
        if not sel_idx:
            messagebox.showwarning("שגיאה", "בחר אפק.")
            return
        
        idx = sel_idx[0]
        entry = self.selected_apks_data[idx]
        
        if not entry.is_url:
            # עבור קבצים מקומיים, קרא את הגודל ישירות מהמערכת
            try:
                # השתמש ב-apk_source אם הוא נתיב לקובץ מקומי אמיתי (לא URL)
                # או ב-temp_local_apk_path אם הוא חולץ מ-ZIP
                file_to_check = entry.temp_local_apk_path if entry.temp_local_apk_path else entry.apk_source
                if file_to_check and os.path.exists(file_to_check):
                    size_b = os.path.getsize(file_to_check)
                    entry.file_size_var.set(self.format_size(size_b))
                    self.selected_apk_size_label_var.set(f"גודל: {entry.file_size_var.get()}")
                    messagebox.showinfo("גודל APK", "גודל קובץ מקומי: " + entry.file_size_var.get())
                else:
                    messagebox.showerror("שגיאה", "לא ניתן למצוא את קובץ ה-APK המקומי.")
                return
            except OSError as e:
                messagebox.showerror("שגיאה", f"לא ניתן לקבל גודל קובץ מקומי: {e}")
                return
        
        # עבור URL, נסה לקבל גודל באמצעות בקשת HTTP
        url = entry.apk_source
        self.selected_apk_size_label_var.set("גודל: מחשב...")
        self.root.update_idletasks() # עדכן את ה-GUI מיד
        
        try:
            # נסה תחילה עם בקשת HEAD כדי לקבל את כותרת 'content-length'
            resp_head = requests.head(url, timeout=10, allow_redirects=True)
            resp_head.raise_for_status() # Raise HTTPError for bad responses (4xx or 5xx)
            
            content_length = resp_head.headers.get('content-length')
            if content_length:
                size_b = int(content_length)
                entry.file_size_var.set(self.format_size(size_b))
                self.selected_apk_size_label_var.set(f"גודל: {entry.file_size_var.get()}")
                messagebox.showinfo("גודל APK", "גודל (מכותרות HTTP): " + entry.file_size_var.get())
                return
            
            # אם אין 'content-length' בכותרות, בצע הורדה זמנית
            messagebox.showinfo("מחשב", "כותרת Content-Length לא זמינה. מנסה הורדה זמנית...")
            with tempfile.TemporaryDirectory() as td:
                temp_apk_path = os.path.join(td, "temp.apk")
                with requests.get(url, stream=True, timeout=60) as r:
                    r.raise_for_status()
                    with open(temp_apk_path, 'wb') as f:
                        shutil.copyfileobj(r.raw, f)
                
                size_b = os.path.getsize(temp_apk_path)
                entry.file_size_var.set(self.format_size(size_b))
                self.selected_apk_size_label_var.set(f"גודל: {entry.file_size_var.get()}")
                messagebox.showinfo("גודל APK", "גודל (מהורדה זמנית): " + entry.file_size_var.get())

        except requests.exceptions.RequestException as e:
            entry.file_size_var.set("שגיאה")
            self.selected_apk_size_label_var.set("גודל: שגיאה")
            messagebox.showerror("שגיאת רשת", "לא ניתן להביא גודל מ-URL: " + str(e))
        except Exception as e:
            entry.file_size_var.set("שגיאה")
            self.selected_apk_size_label_var.set("גודל: שגיאה")
            messagebox.showerror("שגיאה", "אירעה שגיאה: " + str(e))


    def format_size(self, size_bytes):
        if size_bytes < 1024:
            return f"{size_bytes} B"
        elif size_bytes < 1024 * 1024:
            return f"{size_bytes / 1024:.2f} KB"
        else:
            return f"{size_bytes / (1024 * 1024):.2f} MB"

    def clear_apk_details_fields(self):
        self.selected_apk_source_label.config(text="מקור: ---")
        self.selected_apk_appname_entry.delete(0, tk.END)
        self.selected_apk_appname_entry.config(state=tk.DISABLED)
        self.selected_apk_package_entry.delete(0, tk.END)
        self.selected_apk_package_entry.config(state=tk.DISABLED)
        self.selected_apk_description_text.delete("1.0", tk.END)
        self.selected_apk_description_text.config(state=tk.DISABLED)
        self.selected_apk_image_label_var.set("תמונה: כלום")
        self.selected_apk_size_label_var.set("גודל: לא זמין")


    def create_backup_zip_handler(self):
        if not self.selected_apks_data:
            messagebox.showerror("שגיאה", "לא נבחרו APKs.")
            return
        
        output_zip_path = filedialog.asksaveasfilename(
            title="שמור גיבוי ZIP",
            defaultextension=".zip",
            filetypes=(("ZIP", "*.zip"),)
        )
        if not output_zip_path:
            return

        apps_list_for_manifest = [app.to_dict() for app in self.selected_apks_data]
        
        # יצירת אובייקט ה-JSON כולו כמילון פייתון
        store_config_dict = {
            "appList": apps_list_for_manifest,
            "settingsPassword": self.password_var.get(),
            "normalInstallMode": self.normal_install_var.get(),
            "useRootInstall": self.use_root_var.get(),
            "rootInstallMethod": self.root_method_var.get(),
            "installLocation": self.install_location_var.get(),
            "enableSendFeedback": self.send_feedback_var.get(),
            "aboutText": self.about_text_widget.get("1.0", tk.END).strip()
        }
        
        # המר את המילון למחרוזת JSON
        # ensure_ascii=False כדי לאפשר תווים לא-ASCII (כמו עברית) ישירות
        # indent=2 ליצירת קובץ JSON קריא ומסודר
        manifest_json_content = json.dumps(store_config_dict, indent=2, ensure_ascii=False)

        try:
            with zipfile.ZipFile(output_zip_path, 'w', zipfile.ZIP_DEFLATED) as zf:
                # כתוב את קובץ ה-JSON
                # יש צורך בקידוד UTF-8 בגלל ensure_ascii=False
                zf.writestr(MANIFEST_FILE_NAME, manifest_json_content.encode('utf-8'))
                print(f"DEBUG: נכתב {MANIFEST_FILE_NAME} ל-ZIP.")
                
                apks_written = 0
                for app_entry in self.selected_apks_data:
                    # הנתיב האמיתי של קובץ ה-APK אותו נכניס ל-ZIP
                    # אם זה URL, לא נכניס אותו ל-ZIP
                    # אם זה מקומי, נבדוק אם הנתיב הזמני קיים (כלומר יובא מ-ZIP)
                    # אחרת, נשתמש בנתיב המקורי שהמשתמש בחר.
                    actual_apk_path_to_zip = None
                    if not app_entry.is_url:
                        if app_entry.temp_local_apk_path and os.path.exists(app_entry.temp_local_apk_path):
                            actual_apk_path_to_zip = app_entry.temp_local_apk_path
                        elif app_entry.apk_source and os.path.exists(app_entry.apk_source):
                            actual_apk_path_to_zip = app_entry.apk_source
                        
                        if actual_apk_path_to_zip:
                            dest_in_zip = os.path.join(APKS_DIR_NAME, app_entry.apk_file_name)
                            zf.write(actual_apk_path_to_zip, dest_in_zip)
                            apks_written += 1
                            print(f"DEBUG: נוסף APK: {actual_apk_path_to_zip} ל- {dest_in_zip}")
                        else:
                            messagebox.showwarning("חסר APK", f"APK עבור '{app_entry.app_name_var.get()}' לא נמצא ולא ייכלל בגיבוי.")
                    else:
                        print(f"DEBUG: APK עבור '{app_entry.app_name_var.get()}' הוא URL, לא יתווסף ל-ZIP.")
                    
                    # טפל בתמונה באופן דומה
                    # ה- image_path_var הוא הנתיב המקומי המלא של התמונה המוצגת כרגע
                    if app_entry.image_path_var.get() and os.path.exists(app_entry.image_path_var.get()):
                        dest_img_in_zip = os.path.join(IMAGES_DIR_NAME, app_entry.image_file_name_in_zip_var.get())
                        zf.write(app_entry.image_path_var.get(), dest_img_in_zip)
                        print(f"DEBUG: נוספה תמונה: {app_entry.image_path_var.get()} ל- {dest_img_in_zip}")
                    elif app_entry.image_path_var.get(): # אם יש נתיב אבל הקובץ לא קיים
                        messagebox.showwarning("חסרה תמונה", f"תמונה עבור '{app_entry.app_name_var.get()}' לא נמצאה ולא תיכלל בגיבוי.")
                
                print(f"DEBUG: נכתבו {apks_written} קבצי APK ל-ZIP.")
            messagebox.showinfo("הצלחה", f"גיבוי נוצר בהצלחה: {output_zip_path}")
        except Exception as e:
            messagebox.showerror("שגיאה", f"יצירת גיבוי נכשלה: {e}")


    def load_backup_zip_handler(self):
        self.clear_apk_details_fields()
        self.selected_apks_data.clear()
        self.apk_listbox.delete(0, tk.END)
        self._cleanup_temp_dir() # נקה תיקייה זמנית ישנה אם קיימת

        zip_path = filedialog.askopenfilename(title="בחר ZIP לייבוא", filetypes=(("ZIP", "*.zip"),))
        if not zip_path: return

        # צור תיקייה זמנית חדשה עבור הייבוא הנוכחי
        self.current_temp_dir = tempfile.mkdtemp(prefix="kstore_import_")
        print(f"DEBUG: תיקייה זמנית חדשה לייבוא: {self.current_temp_dir}")

        try:
            with zipfile.ZipFile(zip_path, 'r') as zf:
                print(f"DEBUG: קורא ZIP: {zip_path}")
                json_raw_content = None
                try:
                    # קרא את קובץ ה-JSON ופענח אותו
                    json_raw_content = zf.read(MANIFEST_FILE_NAME).decode('utf-8')
                    print(f"DEBUG: תוכן JSON גולמי:\n{json_raw_content[:500]}...") # הדפס התחלה
                    store_config_data = json.loads(json_raw_content)
                    print(f"DEBUG: נתוני JSON שפורשו בהצלחה. מפתחות ראשיים: {list(store_config_data.keys())}")
                except KeyError:
                    messagebox.showerror("שגיאה", f"{MANIFEST_FILE_NAME} לא נמצא ב-ZIP."); self._cleanup_temp_dir(); return
                except json.JSONDecodeError as jde:
                    messagebox.showerror("שגיאה בפענוח JSON", f"לא ניתן לנתח את {MANIFEST_FILE_NAME}: {jde}\nודא שקובץ ה-JSON תקין.")
                    self._cleanup_temp_dir(); return
                
                # טען הגדרות כלליות
                self.password_var.set(store_config_data.get("settingsPassword", "yossi6547"))
                self.normal_install_var.set(store_config_data.get("normalInstallMode", False))
                self.use_root_var.set(store_config_data.get("useRootInstall", True))
                self.root_method_var.set(store_config_data.get("rootInstallMethod", "pm_install"))
                self.install_location_var.set(store_config_data.get("installLocation", "system/app"))
                self.send_feedback_var.set(store_config_data.get("enableSendFeedback", True))
                about_text = store_config_data.get("aboutText", "חנות כשרה.")
                self.about_text_widget.delete("1.0", tk.END); self.about_text_widget.insert(tk.END, about_text)
                print("DEBUG: הגדרות כלליות נטענו.")

                apps_to_load = store_config_data.get("appList", [])
                
                # צור תיקיות יעד לחילוץ בתוך התיקייה הזמנית הראשית
                temp_apks_extract_dir = os.path.join(self.current_temp_dir, APKS_DIR_NAME)
                temp_images_extract_dir = os.path.join(self.current_temp_dir, IMAGES_DIR_NAME)
                os.makedirs(temp_apks_extract_dir, exist_ok=True)
                os.makedirs(temp_images_extract_dir, exist_ok=True)
                
                for app_data in apps_to_load:
                    app_name = app_data.get("name", "Unknown App")
                    apk_file_name = app_data.get("apkFileName") # זה יהיה שם הקובץ בתוך ה-ZIP
                    image_file_name_in_zip = app_data.get("imageFileName") # שם הקובץ של התמונה ב-ZIP
                    apk_url = app_data.get("apkUrl") # יכול להיות None אם זו אפליקציה מקומית
                    package_name = app_data.get("packageName")
                    description = app_data.get("description", f"תיאור עבור {app_name}")
                    file_size_bytes = int(app_data.get("fileSize", 0))

                    is_url_app = (apk_url is not None and apk_url != "") # True אם apk_url לא None ולא ריק

                    initial_apk_source = apk_url if is_url_app else "" # יתעדכן בהמשך לנתיב זמני אם יחולץ

                    print(f"DEBUG: טוען אפליקציה: '{app_name}', apkFileName: '{apk_file_name}', is_url: {is_url_app}, apkUrl: {apk_url}")

                    app_entry = AppEntry(initial_apk_source, is_url=is_url_app)
                    app_entry.app_name_var.set(app_name)
                    app_entry.package_name_var.set(package_name if package_name else app_entry._generate_package_name())
                    app_entry.description_var.set(description)
                    app_entry.file_size_var.set(self.format_size(file_size_bytes))
                    
                    if apk_file_name: # תמיד אמור להיות apk_file_name, בין אם מקומי או URL
                        app_entry.apk_file_name = apk_file_name # שם הקובץ כפי שמופיע במניפסט
                        
                        if not is_url_app: # אם זו אפליקציה מקומית, נחלץ אותה
                            apk_path_in_zip = os.path.join(APKS_DIR_NAME, apk_file_name)
                            temp_local_apk_path = os.path.join(temp_apks_extract_dir, apk_file_name)
                            print(f"DEBUG:  מנסה לחלץ APK '{apk_path_in_zip}' אל '{temp_local_apk_path}'")
                            try:
                                # חשוב: zf.extract חולץ ביחס לתיקייה הנוכחית של ה-zipfile,
                                # ולכן צריך לציין את התיקייה הזמנית כנתיב השני.
                                zf.extract(apk_path_in_zip, self.current_temp_dir) 
                                app_entry.temp_local_apk_path = temp_local_apk_path
                                app_entry.apk_source = temp_local_apk_path # המקור עכשיו הוא הקובץ הזמני
                                print(f"DEBUG:  APK חולץ בהצלחה: {temp_local_apk_path}")
                            except KeyError:
                                print(f"אזהרה: קובץ APK '{apk_file_name}' לא נמצא ב-ZIP עבור '{app_name}'.")
                                messagebox.showwarning("קובץ חסר", f"קובץ APK '{apk_file_name}' לא נמצא ב-ZIP עבור '{app_name}'.")
                                app_entry.apk_source = None # אין קובץ מקומי
                            except Exception as e:
                                print(f"שגיאה בחילוץ APK '{apk_file_name}': {e}")
                                messagebox.showerror("שגיאת חילוץ", f"שגיאה בחילוץ APK '{apk_file_name}': {e}")
                                app_entry.apk_source = None # אין קובץ מקומי
                        
                    if image_file_name_in_zip:
                        image_path_in_zip = os.path.join(IMAGES_DIR_NAME, image_file_name_in_zip)
                        temp_local_image_path = os.path.join(temp_images_extract_dir, image_file_name_in_zip)
                        print(f"DEBUG:  מנסה לחלץ תמונה '{image_path_in_zip}' אל '{temp_local_image_path}'")
                        try:
                            zf.extract(image_path_in_zip, self.current_temp_dir)
                            app_entry.temp_local_image_path = temp_local_image_path
                            app_entry.image_path_var.set(temp_local_image_path)
                            app_entry.image_file_name_in_zip_var.set(image_file_name_in_zip)
                            print(f"DEBUG:  תמונה חולצה בהצלחה: {temp_local_image_path}")
                        except KeyError:
                            print(f"אזהרה: תמונה '{image_file_name_in_zip}' לא נמצאה ב-ZIP עבור '{app_name}'.")
                            messagebox.showwarning("קובץ חסר", f"תמונה '{image_file_name_in_zip}' לא נמצאה ב-ZIP עבור '{app_name}'.")
                        except Exception as e:
                            print(f"שגיאה בחילוץ תמונה '{image_file_name_in_zip}': {e}")
                            messagebox.showerror("שגיאת חילוץ", f"שגיאה בחילוץ תמונה '{image_file_name_in_zip}': {e}")
                    
                    self.selected_apks_data.append(app_entry)
                    self.apk_listbox.insert(tk.END, app_entry.apk_file_name)
            
            if self.selected_apks_data:
                self.apk_listbox.selection_clear(0, tk.END)
                self.apk_listbox.selection_set(0) # בחר את הפריט הראשון
                self.on_apk_select_from_list(None)
            
            messagebox.showinfo("ייבוא הושלם", f"גיבוי '{os.path.basename(zip_path)}' יובא בהצלחה.")

        except zipfile.BadZipFile:
            messagebox.showerror("שגיאה", "קובץ ה-ZIP פגום או לא חוקי.")
            self._cleanup_temp_dir()
        except Exception as e:
            messagebox.showerror("שגיאה", f"טעינת גיבוי ZIP נכשלה: {e}")
            self._cleanup_temp_dir()

# --- נקודת כניסה ליישום ---
if __name__ == "__main__":
    root = tk.Tk()
    app = BackupCreatorApp(root)
    root.mainloop()